@@ -2,7 +2,7 @@
 #include "c.h"
 #endif
 
-int b_fun(){
+int b_fun(void){
 #if defined(WITH_C)
 return c_fun();
 #else
